#include "OgreChildPSystemRenderingRun.h"
#include "OgreIlluminationManager.h"
#include "OgreDistanceCubeMapRenderingRun.h"



OgreChildPSystemRenderingRun::OgreChildPSystemRenderingRun(OgreSharedRuns* sharedRuns,
															String name,
															unsigned long startFrame,
															unsigned long updateInterval,
															unsigned int resolution,
															bool perspectiveRendering,
															String childParticleSystemName,
															String particleScriptName,
															bool useOwnMaterial,
															String materialName)
														   :ChildPsystemRenderingRun(resolution, perspectiveRendering, startFrame, updateInterval)
														   , OgreRenderingRun(startFrame, updateInterval)
														   , RenderingRun(startFrame, updateInterval)
{
	this->sharedRuns = sharedRuns;
	this->name = name;	
	this->resolution = resolution;
	this->materialName = materialName;
	this->particleScriptName = particleScriptName;
	this->perspectiveRendering = perspectiveRendering;
	this->childParticleSystemName = childParticleSystemName;
	this->useOwnMaterial = useOwnMaterial;
	createImpostorTexture();	
}

void OgreChildPSystemRenderingRun::createImpostorTexture()
{
	TexturePtr texPtr = Ogre::TextureManager::getSingleton().createManual(name, 
																		"default",
																		TEX_TYPE_2D,
																		resolution,
																		resolution,
																		0,
																		0,
																		PF_FLOAT16_RGBA,
																		TU_RENDERTARGET);
	 impostorTexture = texPtr.getPointer();
	 impostorCamera = Root::getSingleton()._getCurrentSceneManager()->createCamera(name + "_CAMERA");
	 //add viewport to rendertarget
	 HardwarePixelBuffer* hpb = (impostorTexture->getBuffer()).getPointer();
	 RenderTarget* rt = hpb->getRenderTarget();
	 Viewport* v = rt->addViewport(impostorCamera);
	 v->setBackgroundColour(ColourValue(0,1,0,0));
	 v->setOverlaysEnabled(false);
	 rt->setAutoUpdated(false);
}

void OgreChildPSystemRenderingRun::updateFrame(unsigned long frameNum)
{
	
	Material* mat = (Material*) MaterialManager::getSingleton().getByName(materialName).getPointer();

    SceneManager* sm = Ogre::Root::getSingleton()._getCurrentSceneManager();
	RenderQueue* rq = sm->getRenderQueue();
	rq->clear();

	sharedRuns->getRoot()->updateBounds();
	Sphere boundSphere = ((OgreSharedRuns*)sharedRuns->getRoot(ILLUMRUN_HPP_IMPOSTOR))->getBoundingSphere();
	Camera* mainCam = OgreIlluminationManager::getSingleton().getMainCamera();
	impostorCamera->setPosition(mainCam->getPosition());
	Vector3 dir = boundSphere.getCenter() - mainCam->getPosition();
	float r = boundSphere.getRadius();
	r = 200;
	float d = dir.normalise();
	impostorCamera->setDirection(dir);

	if(perspectiveRendering)
	{
		impostorCamera->setProjectionType(PT_PERSPECTIVE);
		impostorCamera->setAspectRatio(1.0);
		Radian a = Math::ASin(r / d) * 2.0;
		impostorCamera->setFOVy(/*a*/ Radian(Degree(90)));
		impostorCamera->setNearClipDistance(/*std::max(0.01f, d - r)*/0.1);
		impostorCamera->setFarClipDistance(/*r + d*/1000);		
	}
	else
	{
		impostorCamera->setPosition(boundSphere.getCenter());
		impostorCamera->setProjectionType(PT_ORTHOGRAPHIC);
		Matrix4 m;
		m = Matrix4::IDENTITY;
		float scale = 1.0 / boundSphere.getRadius();
		m.setScale(Vector3(scale, scale, scale));
		impostorCamera->setCustomProjectionMatrix(true, m);
	}	

	ParticleSystem* psys = Root::getSingleton()._getCurrentSceneManager()->getParticleSystem(childParticleSystemName);
	psys->getParentSceneNode()->setPosition(boundSphere.getCenter());
	psys->_notifyCurrentCamera(impostorCamera);
	psys->_updateRenderQueue(rq);
	
	if(!useOwnMaterial)
		setMaterialForRenderables(materialName,rq);
	
	RenderTarget* rt = impostorTexture->getBuffer().getPointer()->getRenderTarget();
	 
	rt->update();	
	
	
	static int framecount = 0;
	String filename = "impostor";
	filename.append(this->name);
	//filename.append(StringConverter::toString(framecount));
	filename.append(".dds");
	rt->writeContentsToFile(filename);
	framecount++;	
	
	if(!useOwnMaterial)
		restoreMaterials();
}

